{{Breakout board Seven-segment display object. Controls six digits on PPDB and generates
  buzzer signal on buzzpin at buzzfreq (set below) at all times, since the outputs on the
  chip are logically 'OR'ed together ... all another cog has to do is write a '1' to the
  same pin to stop the buzzing.

    Author: Steve Nicholson
   Revisor: Tim Meyers for v 1.1 forward
   Version: 1.0  ( 4 Jan 2007)
   Version: 1.1  (18 Jan 2010) Added SetDigits method
   Version: 1.2  (12 Feb 2010) rewired with CD-4511 decoding propeller value
   Version: 1.3  (22 Jul 2010) revised for break board w/buzzer.
   Author Email: ssteve@mac.com
   Revisor Email: tmeyers33@comcast.net
   Copyright: none

}}

VAR
  long Ldig, Hdig                                       ' The pins for LED cathodes.
  long D0Pin, D3Pin                                     ' The pins for CD4511.
  long flags                                            ' control flags
  long myStack[10]                                      ' stack for cog
  long runningCogID                                     ' cog id holding ShowValue routine
  long myValue                                          ' whats being displayed now!
  byte DD                                               ' Displayed digits
  long p1,p2                                            ' temp vars
  long sweep
  
CON
  isEnabled = %0001
  isShutOff = %0010
  BuzzPin  = 0
  BuzzFreq = 4000

OBJ
  sw    :       "squarewave"
  
PUB Stop                                                ' make sure 'stop' is 1st pub method
'' Stop the display                                     ' to prevent execution as pgm.
   ' see ShowValue method for flag use ... when sutting down, wait for flags
   ' to zero (0).  Then remove from chip guts.
  
  if runningCogID
    flags := isShutOff
    repeat until flags == 0                             ' wait for acknowledge
    outa[Ldig..D3Pin]~~
    cogstop(runningCogID~ - 1)                          ' pull it from the cog

PUB Start
'' Start the display
''   disable - the initial enabled state
''   pass cog ctra settings for buzzpin using spec regs #8 & #10.
  myValue := 0
  Ldig := 1
  Hdig := 6                
  SetDigits(6)
  D0Pin := 7
  D3Pin := 7 + 3
  disable
  stop
  sweep := 500
  dira[BuzzPin]~~
  outa[BuzzPin]~~                                       ' disable from cog0 running spin
  sw.freq(0,BuzzPin,BuzzFreq)                           ' compute settings using spin
  p1 := spr[8]~                                         ' clr after reading values
  p2 := spr[10]~
  runningCogID := cognew(ShowValue(@p1), @myStack) + 1  ' pass timer values
  SetValue(1)
  SetValue(0)

PUB SetDigits(digits)       '' allows # digits to change on the fly
    DD := digits

PUB Enable                                          '' Enable the display
  flags |= isEnabled

PUB Disable                                         '' Disable the display
  flags &= !isEnabled

PUB SetSweep(aValue)
  sweep := aValue
  
PUB Chirp
  outa[BuzzPin]~                                        ' buzzer on!
  waitcnt(clkfreq / 50 + cnt)                           ' the delay value adjust
  outa[BuzzPin]~~                                       ' buzzer off!

PUB Beep(ms)
  outa[BuzzPin]~                                        ' buzzer on!
  waitcnt((clkfreq / 1000) * ms + cnt)                  ' the adjust
  outa[BuzzPin]~~                                       ' buzzer off!

PUB pause(ms)
  waitcnt((clkfreq / 1000) * ms + cnt)                  'the delay value adjust

PUB SetValue(theValue) | divisor, dgts            '' Set the value to display
    case theValue
               10..99: SetDigits(2)
             100..999: SetDigits(3)
           1000..9999: SetDigits(4)
         10000..99999: SetDigits(5)
       100000..999999: SetDigits(6)
       other         : SetDigits(1)
    myValue := theValue
    if (myValue => 0)
      Enable 
    else                
      Disable                                       ' dont show negatives at present

PRI ShowValue(cta) | cathode, divisor, displayValue
  '' ShowValue runs in its own cog and continually updates the display
  ' This routine has control over the output pins.  myValue, flags and DD are the only
  ' inputs this routine looks at from outside this object.  When started the
  ' buzzpin is activated in timer control reg A, it runs continuosly at BuzzFreq and
  ' is controlled by writing a '1' from another cog to turn it off.  To turn it on,
  ' you write a '0' to the BuzzPin.
  outa[BuzzPin]~                                        ' write '0'
  dira[BuzzPin]~~                                       ' prog as output from this cog
  spr[08] := long[cta + 0]                              ' set spec register values passed
  spr[10] := long[cta + 4]                              ' from calling side cog 
  repeat
    if flags & isEnabled
      displayValue := myValue                           ' use a 'copy' of the number to show
      divisor := 1                                      ' divisor is used to isolate a digit to display
      Ldig := Hdig - DD + 1                             ' start with lowest digit
      repeat cathode from Hdig to Ldig                  ' only display as many digits as there are pins
        outa[D3Pin..D0Pin] := (displayValue / divisor) // 10    ' present value to 4511
        outa[cathode]~                                  ' turn cathode on (write '0')
        dira[D0Pin..D3Pin]~~                            ' send 4511 BCD digit
        dira[cathode]~~                                 ' turn off cathode
        waitcnt(clkfreq / sweep + cnt)                    ' the delay value can be tweaked to adjust
        dira[Ldig..Hdig]~                               ' turn all cathodes off
        divisor *= 10                                   ' adj div
    elseif flags & isShutOff
      dira[Hdig..Ldig]~                                 ' disable all digits
      dira[D0Pin..D3Pin]~                               ' disable all data
      dira[BuzzPin]~                                    ' disable buzz pin
      ctra~                                             ' disable counter
      flags~                                            ' clear flags (see STOP method)
                                                        '  tells spin, all done!
      repeat                                            ' hold for removal
    else
      dira[Hdig..Ldig]~                                 'disable all digits
      waitcnt (clkfreq / 10 + cnt)                      'wait 1/10 second before checking again
     
DAT
        'Common cathode 7-segment displays are activated by bringing the cathode to ground
      {DigSel          byte    %11111110
                        byte    %11111101
                        byte    %11111011
                        byte    %11110111
                        byte    %11101111
                        byte    %11011111
                        byte    %10111111
                        byte    %01111111}


        
       {Digit0          byte    %00111111      ' no longer needed because of 4511
        Digit1          byte    %00000110
        Digit2          byte    %01011011      ' but not deleted
        Digit3          byte    %01001111
        Digit4          byte    %01100110      ' decoding by 4511 frees up 3 pins in the CPU
        Digit5          byte    %01101101
        Digit6          byte    %01111101      ' plus improves current in LED's w/+5v pwr.
        Digit7          byte    %00000111
        Digit8          byte    %01111111      ' LED's are nice and bright!
        Digit9          byte    %01100111}     